package com.dgtly.sync.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.dgtly.common.exception.BusinessException;
import com.dgtly.common.utils.ShiroSaltUtil;
import com.dgtly.common.utils.http.HttpUtils;
import com.dgtly.common.utils.security.EncryptPassWordClass;
import com.dgtly.sync.domain.Customers;
import com.dgtly.sync.domain.SyncFailDetail;
import com.dgtly.sync.domain.SyncLog;
import com.dgtly.sync.mapper.CustomersMapper;
import com.dgtly.sync.mapper.SyncFailDetailMapper;
import com.dgtly.sync.utils.CustomerAccessTokenUtil;
import com.dgtly.sync.utils.CustomerAccessTokenUtil.UrlType;
import com.dgtly.system.domain.SysUser;
import com.dgtly.system.domain.SysUserExt;
import com.dgtly.system.mapper.SysUserExtMapper;
import com.dgtly.system.mapper.SysUserMapper;
import com.dgtly.system.service.ISysConfigService;
import com.dgtly.system.service.impl.SysUserServiceImpl;
import com.dgtly.wxportal.domain.WxQyUser;
import com.dgtly.wxportal.mapper.WxQyUserMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

@Component
public class AnalysisDiyCustomerComponent {

    private static final Logger log = LoggerFactory.getLogger(AnalysisDiyCustomerComponent.class);


    @Autowired
    private CustomerAccessTokenUtil customerAccessTokenUtil;

    @Autowired
    private CustomersMapper customersMapper;
    @Autowired
    private SysUserServiceImpl sysUserService;
    @Autowired
    private SysUserExtMapper sysUserExtMapper;
    @Autowired
    private SysUserMapper sysUserMapper;
    @Autowired
    private WxQyUserMapper wxQyUserMapper;

    @Autowired
    private ISysConfigService configService;
    @Autowired
    private ISyncLogService syncLogService;
    @Autowired
    private SyncFailDetailMapper syncFailDetailMapper;
    private Long deptId;
    private Long[] roleIds;
    private Long[] roleGuideIds;
    private Long[] roleCwIds;
    private Long[] roleCwDiyIds;
     /**
     * 获取并解析DIY经销商用户
     * 因为经销商用户并不涉及用信息更新等方面 所以只插入不更新
     * @return
     */
    //@Override
    public SyncLog analyDiyCustomer() {
        if(deptId==null){
            deptId = Long.parseLong(configService.selectConfigByKey("libang.customer.deptid"));
        }
        if(roleIds==null){
            Long roleId = Long.parseLong(configService.selectConfigByKey("libang.customer.roleid"));
            roleIds = new Long[]{roleId};
        }

        if (roleGuideIds == null) {
            Long roleId = Long.parseLong(configService.selectConfigByKey("libang.guide.roleid"));
            roleGuideIds = new Long[]{roleId};
        }
        if (roleCwIds == null) {
            Long roleId = Long.parseLong(configService.selectConfigByKey("libang.cw.roleid"));
            roleCwIds = new Long[]{roleId};
        }
        if (roleCwDiyIds == null) {
            Long roleId = Long.parseLong(configService.selectConfigByKey("libang.cw.roleid"));
            Long roleDiyId = Long.parseLong(configService.selectConfigByKey("libang.guide.roleid"));
            roleCwDiyIds = new Long[]{roleId, roleDiyId};
        }

        List<SyncFailDetail> failDetails = new ArrayList<>();
        SyncLog syncLog = new SyncLog();
        syncLog.setStartTime(new Date());
        String packageId = "diycustomer"+syncLog.getStartTime().getTime();
        syncLog.setPackageId(packageId);
        long total = 0;
        long success = 0;
        Set<String> loginNameSet = sysUserService.selectAllUserLoginName();
        Set<String> diyNameSet = sysUserService.selectDiyUserLoginName();
        String managerJson ="";
        try{
            managerJson = HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.MANAGER,null));
        }catch (Exception e){
            log.error("获取经销商经理接口错误！");
            customerAccessTokenUtil.reSetToken();
            throw e;
        }

        JSONObject mjson = JSONObject.parseObject(managerJson);
        Map<String,Customers> codeMap = customersMapper.selectCustomerMapByCode();

        Map<String,Customers> chainsMap = customersMapper.selectCustomerMapByChainsCode();
        int code = mjson.getInteger("code");
        if(code==0){
            success+=buildSysUser(mjson,loginNameSet,1,failDetails,packageId,chainsMap);
            total+=mjson.getJSONArray("userlist").size();
        }



        String guideJson ="";
        try{
            guideJson = HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.STOREGUIDE,null));
        }catch (Exception e){
            log.error("获取经销商导购接口错误！");
            customerAccessTokenUtil.reSetToken();
            throw e;
        }

        JSONObject gjson = JSONObject.parseObject(guideJson);
        code = gjson.getInteger("code");
        if(code==0){
            success+=buildSysUser(gjson,loginNameSet,0,failDetails,packageId,codeMap);
            total+=gjson.getJSONArray("userlist").size();
        }

        String financeJson ="";
        try{
            financeJson = HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.FINANCE,1+""));
        }catch (Exception e){
            log.error("获取经销商财务接口错误！");
            customerAccessTokenUtil.reSetToken();
            throw e;
        }

        JSONObject fjson = JSONObject.parseObject(financeJson);
        Integer fcount = fjson.getInteger("count");
        if(fcount>100){
            int co = 0;
            if (fcount%100 != 0){
                co = (fcount/100) +2;
            }else{
                co = (fcount/100) +1;
            }
            for (int i = 1; i < co; i++) {
                try {
                    fjson = JSONObject.parseObject(HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.FINANCE,i+"")));
                } catch (Exception e) {
                    log.error("循环获取经销商财务接口错误！第"+i+"页");
                    customerAccessTokenUtil.reSetToken();
                    throw e;
                }
                code = fjson.getInteger("code");
                if(code==0){
                    success+=buildSysUser(fjson,loginNameSet,2,failDetails,packageId,chainsMap);
                    total+=fjson.getJSONArray("userlist").size();
                }
            }
        }else{
            code = fjson.getInteger("code");
            if(code==0){
                success+=buildSysUser(fjson,loginNameSet,2,failDetails,packageId,chainsMap);
                total+=fjson.getJSONArray("userlist").size();
            }
        }


        String warehouseJson ="";
        try{
            warehouseJson = HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.WAREHOUSE,1+""));
        }catch (Exception e){
            log.error("获取经销商仓管接口错误！");
            customerAccessTokenUtil.reSetToken();
            throw e;
        }

        JSONObject wjson = JSONObject.parseObject(warehouseJson);
        Integer wcount = wjson.getInteger("count");
        if(wcount>100){
            int co = 0;
            if (wcount%100 != 0){
                co = (wcount/100) +2;
            }else{
                co = (wcount/100) +1;
            }

            for (int i = 1; i < co; i++) {
                try {
                    wjson = JSONObject.parseObject(HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.WAREHOUSE,i+"")));
                } catch (Exception e) {
                    log.error("获取经销商仓管接口错误！第"+i+"页");
                    customerAccessTokenUtil.reSetToken();
                    throw e;
                }
                code = wjson.getInteger("code");
                if(code==0){
                    success+=buildSysUser(wjson,loginNameSet,3,failDetails,packageId,chainsMap);
                    total+=wjson.getJSONArray("userlist").size();
                }
            }
        }else{
            code = wjson.getInteger("code");
            if(code==0){
                success+=buildSysUser(wjson,loginNameSet,3,failDetails,packageId,chainsMap);
                total+=wjson.getJSONArray("userlist").size();
            }
        }

        try {
            JSONObject bossListJson = JSONObject.parseObject(HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.BOSSLIST, null)));
            code = bossListJson.getInteger("code");
            if (code == 0) {
                success += buildSysUserEbr(bossListJson, loginNameSet, diyNameSet, 4, failDetails, packageId, chainsMap);
                total += bossListJson.getJSONArray("userlist").size();
            }
        } catch (Exception e) {
            log.error("获取厨卫老板列表接口错误！");
            customerAccessTokenUtil.reSetToken();
            throw e;
        }

        try {
            JSONObject bossListJson = JSONObject.parseObject(HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.MANAGERLIST, null)));
            code = bossListJson.getInteger("code");
            if (code == 0) {
                success += buildSysUserEbr(bossListJson, loginNameSet, diyNameSet, 5, failDetails, packageId, chainsMap);
                total += bossListJson.getJSONArray("userlist").size();
            }
        } catch (Exception e) {
            log.error("获取厨卫经理列表接口错误！");
            customerAccessTokenUtil.reSetToken();
            throw e;
        }

        try{
            syncLog.setTotalCount(total);
            syncLog.setSuccessCount(success);
            syncLog.setFaileCount((long)failDetails.size());
            syncLog.setEndTime(new Date());
            syncLog.setSyncType("diycustomer");

            if(failDetails.size()>0){
                syncFailDetailMapper.batchInsertSyncFailDetail(failDetails);
            }
        }catch (Exception e){
            log.error("解析用户插入解析日志出错",e);
            e.printStackTrace();
            throw new BusinessException("解析用户插入解析日志出错",e);
        }finally {
            syncLogService.insertSyncLog(syncLog);
        }


        return null;
    }


    /**
     * 构建 插入用户List
     * @param json
     * @param loginNameSet 排重用
     * @param isManager
     * @return
     */
    public Integer buildSysUser(JSONObject json, Set<String> loginNameSet, int isManager, List<SyncFailDetail> failDetails,String packageId, Map<String,Customers> cusMap){
        JSONArray jsonArray = json.getJSONArray("userlist");
        int count = 0;

        for(int i =0;i<jsonArray.size();i++){
            JSONObject jo = jsonArray.getJSONObject(i);

            try {
                String userNo = jo.getString("userNo");
                String enable = jo.getString("enable");
                String name = jo.getString("name");
                String status = jo.getString("status");
                Boolean isSync = jo.getBoolean("isSync");
                Boolean isDelete = jo.getBoolean("isDelete");
                String userid = jo.getString("userid");
                String customerCode = jo.getString("depCode");
                /*企业微信中成员启用状态。1表示启用的成员，0表示被禁用*/
                /*if(enable==null||enable.trim().equals("0")){
                    continue;
                }
                *//*企业微信中人员激活状态: 1=已激活，2=已禁用，4=未激活，5=退出企业。*//*
                if(status==null || !status.trim().equals("1")){
                    continue;
                }
                *//*人员信息是否已更新入企业微信。true：更新成功false：更新失败*//*
                if(!isSync){
                    continue;
                }
                *//*人员信息是否被标记删除true：删除，false：未删除*//*
                if(isDelete){
                    continue;
                }*/
                if(userid==null||userid.trim().equals("")){
                    continue;
                }
                if (!loginNameSet.contains(userid)) {

                    /** =============用户基本信息构建 start===============*/
                    SysUser user = new SysUser();
                    user.setCompanyId(1L);
                    user.setDeptId(deptId);
                    user.setLoginName(userid);
                    user.setUserName(jo.getString("name"));
                    user.setPhonenumber(jo.getString("mobile"));

                    if (isManager == 0) {
                        user.setRoleIds(roleGuideIds);
                    } else {
                        user.setRoleIds(roleIds);
                    }
                    user.setPassword(user.getLoginName());
                    user.setSalt(ShiroSaltUtil.randomSalt());
                    user.setPassword(EncryptPassWordClass.encryptPassword(user.getLoginName(), user.getPassword(), user.getSalt()));


                    /** =============用户基本信息构建 end===============*/


                    /**======== 构建diy用户扩展信息表 start===============*/
                    SysUserExt userExt = new SysUserExt();
                    userExt.setSalesLevel("customer_level");
                    userExt.setCustomerCode(jo.getString("depCode"));
                    userExt.setIsCustomerManager(isManager + "");
                    userExt.setSapEmployeeId(userNo);

                    userExt.setPostName("DIY经销商");
                    String orgCode = jo.getString("depCode");
                    if(cusMap.containsKey(orgCode)){
                        Customers c = cusMap.get(orgCode);
                        userExt.setOrgCode(c.getChainsCode());
                        userExt.setOrgName(c.getChainsName());
                    }else{
                        userExt.setOrgCode(orgCode);
                    }

//                    /**======== 构建diy用户wx信息表 start===============*/
//                    WxQyUser wxQyUser = new WxQyUser();
//                    wxQyUser.setUserid(jo.getString("userid"));
//                    wxQyUser.setMobile(jo.getString("mobile"));
                    user.setSysUserExt(userExt);
                    insertUserAndExt(user);
                    count++;
                    loginNameSet.add(userid);
                }else{
                    /*已经存在的，需要先清空职位，更新职位
                    （首先查询当前用户职位，判断当前职位是否包含当前职位，不存在就更新，存在就不更新）*/
                    SysUser user = sysUserService.selectUserByLoginName(userid);

                    if (!user.getSysUserExt().getIsCustomerManager().contains(isManager + "") ||
                            !user.getSysUserExt().getCustomerCode().equals(customerCode) || !user.getSysUserExt().getOrgCode().equals(customerCode)) {
                        if (!user.getSysUserExt().getIsCustomerManager().contains(isManager + "")) {
                            /*更新职位*/
                            user.getSysUserExt().setIsCustomerManager(user.getSysUserExt().getIsCustomerManager() + "," + isManager);
                        }
                        if (!user.getSysUserExt().getCustomerCode().equals(customerCode) || !user.getSysUserExt().getOrgCode().equals(customerCode)) {
                            /*更新经销商code*/
                            if (cusMap.containsKey(customerCode)) {
                                Customers c = cusMap.get(customerCode);
                                user.getSysUserExt().setOrgCode(c.getChainsCode());
                                user.getSysUserExt().setOrgName(c.getChainsName());
                            } else {
                                user.getSysUserExt().setOrgCode(customerCode);
                            }
                        }
                        log.info("updateSysUserExt" + customerCode);
                        sysUserExtMapper.updateSysUserExt(user.getSysUserExt());
                    }
                    /*解决名称不一样的情况*/
                    if(!user.getUserName().equals(name)){
                        user.setUserName(name);
                        sysUserService.updateUserInfo(user);
                    }
                }
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
                SyncFailDetail syncFailDetail = new SyncFailDetail();
                syncFailDetail.setPackageId(packageId);
                syncFailDetail.setFailReason(e.getMessage());
                syncFailDetail.setDataJson(jo.toJSONString());
                syncFailDetail.setFailLevel("1");
                syncFailDetail.setExceptionType(e.getClass().getSimpleName());
                failDetails.add(syncFailDetail);
            }
        }

        return count;
    }


    /**
     * 构建 插入用户List
     * @param json
     * @param loginNameSet 排重用
     * @param isManager
     * @return
     */
    public Integer buildSysUserEbr(JSONObject json, Set<String> loginNameSet,Set<String> diyNameSet, int isManager, List<SyncFailDetail> failDetails,String packageId, Map<String,Customers> cusMap){
        JSONArray jsonArray = json.getJSONArray("userlist");
        int count = 0;
        for(int i =0;i<jsonArray.size();i++){
            JSONObject jo = jsonArray.getJSONObject(i);

            try {
                String userNo = jo.getString("userNo");
                String name = jo.getString("name");
                String userid = jo.getString("userid");
                String customerCode = jo.getString("depCode");
                /*企业微信中成员启用状态。1表示启用的成员，0表示被禁用*/
                /*if(enable==null||enable.trim().equals("0")){
                    continue;
                }
                *//*企业微信中人员激活状态: 1=已激活，2=已禁用，4=未激活，5=退出企业。*//*
                if(status==null || !status.trim().equals("1")){
                    continue;
                }
                *//*人员信息是否已更新入企业微信。true：更新成功false：更新失败*//*
                if(!isSync){
                    continue;
                }
                *//*人员信息是否被标记删除true：删除，false：未删除*//*
                if(isDelete){
                    continue;
                }*/
                if(userid==null||userid.trim().equals("")){
                    continue;
                }
                if (!loginNameSet.contains(userid)) {

                    /** =============用户基本信息构建 start===============*/
                    SysUser user = new SysUser();
                    user.setCompanyId(1L);
                    user.setDeptId(deptId);
                    user.setLoginName(userid);
                    user.setUserName(jo.getString("name"));
                    user.setPhonenumber(jo.getString("mobile"));
                    if (diyNameSet.contains(userid)) {
                        user.setRoleIds(roleCwDiyIds);
                    } else {
                        user.setRoleIds(roleCwIds);
                    }
                    user.setPassword(user.getLoginName());
                    user.setSalt(ShiroSaltUtil.randomSalt());
                    user.setPassword(EncryptPassWordClass.encryptPassword(user.getLoginName(), user.getPassword(), user.getSalt()));


                    /** =============用户基本信息构建 end===============*/


                    /**======== 构建diy用户扩展信息表 start===============*/
                    SysUserExt userExt = new SysUserExt();
                    userExt.setSalesLevel("customer_level");

                    userExt.setIdentity("2");// 1=diy,2=ebr,3=all

                    userExt.setCustomerCode(jo.getString("depCode"));
                    userExt.setIsCustomerManager(isManager + "");
                    userExt.setSapEmployeeId(userNo);

                    userExt.setPostName("CW经销商");
                    String orgCode = jo.getString("depCode");
                    if(cusMap.containsKey(orgCode)){
                        Customers c = cusMap.get(orgCode);
                        userExt.setOrgCode(c.getChainsCode());
                        userExt.setOrgName(c.getChainsName());
                    }else{
                        userExt.setOrgCode(orgCode);
                    }

//                    /**======== 构建diy用户wx信息表 start===============*/
//                    WxQyUser wxQyUser = new WxQyUser();
//                    wxQyUser.setUserid(jo.getString("userid"));
//                    wxQyUser.setMobile(jo.getString("mobile"));
                    user.setSysUserExt(userExt);
                    insertUserAndExt(user);
                    count++;
                    loginNameSet.add(userid);
                }else{
                    /*已经存在的，需要先清空职位，更新职位
                    （首先查询当前用户职位，判断当前职位是否包含当前职位，不存在就更新，存在就不更新）*/
                    SysUser user = sysUserService.selectUserByLoginName(userid);

                    if (!user.getSysUserExt().getIsCustomerManager().contains(isManager + "") ||
                            !user.getSysUserExt().getCustomerCode().equals(customerCode) || !user.getSysUserExt().getOrgCode().equals(customerCode)) {
                        if (!user.getSysUserExt().getIsCustomerManager().contains(isManager + "")) {
                            /*更新职位*/
                            user.getSysUserExt().setIsCustomerManager(user.getSysUserExt().getIsCustomerManager() + "," + isManager);
                        }
                        if (!user.getSysUserExt().getCustomerCode().equals(customerCode) || !user.getSysUserExt().getOrgCode().equals(customerCode)) {
                            /*更新经销商code*/
                            if (cusMap.containsKey(customerCode)) {
                                Customers c = cusMap.get(customerCode);
                                user.getSysUserExt().setOrgCode(c.getChainsCode());
                                user.getSysUserExt().setOrgName(c.getChainsName());
                            } else {
                                user.getSysUserExt().setOrgCode(customerCode);
                            }
                        }
                        log.info("updateSysUserExt" + customerCode);
                        sysUserExtMapper.updateSysUserExt(user.getSysUserExt());
                    }
                    /*解决名称不一样的情况*/
                    if(!user.getUserName().equals(name)){
                        user.setUserName(name);
                        sysUserService.updateUserInfo(user);
                    }
                    //更新 identity 1=diy,2=ebr,3=all
                    if (diyNameSet.contains(userid) && !"3".equals(user.getSysUserExt().getIdentity())) {
                        log.info("updateSysUserExt identity 3" + userid);
                        user.getSysUserExt().setIdentity("3");
                        sysUserExtMapper.updateSysUserExt(user.getSysUserExt());
                    }
                    if (!diyNameSet.contains(userid) && !"2".equals(user.getSysUserExt().getIdentity())) {
                        log.info("updateSysUserExt identity 2" + userid);
                        user.getSysUserExt().setIdentity("2");
                        sysUserExtMapper.updateSysUserExt(user.getSysUserExt());
                    }
                }
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
                SyncFailDetail syncFailDetail = new SyncFailDetail();
                syncFailDetail.setPackageId(packageId);
                syncFailDetail.setFailReason(e.getMessage());
                syncFailDetail.setDataJson(jo.toJSONString());
                syncFailDetail.setFailLevel("1");
                syncFailDetail.setExceptionType(e.getClass().getSimpleName());
                failDetails.add(syncFailDetail);
            }
        }

        return count;
    }


    @Transactional(rollbackFor = Exception.class)
    public Integer insertUserAndExt(SysUser user) {
        // 新增用户信息
        int rows = sysUserMapper.insertUser(user);

        // 新增用户与角色管理
        sysUserService.insertUserRole(user);
        SysUserExt userExt = user.getSysUserExt();
        userExt.setUserId(user.getUserId());

        int i1 = sysUserExtMapper.insertOrUpSysUserExt(userExt);


        //wxQyUser.setSysUserId(user.getUserId());
//        int i2 =wxQyUserMapper.insertWxQyUser(wxQyUser);
//        if(i1+i2<2){
//            throw new RuntimeException("插入UserExt或WxUser错误");
//        }
        return rows;
    }
}
