package com.dgtly.system.mapper;

import java.util.List;
import java.util.Map;
import java.util.Set;

import com.dgtly.system.domain.SysUser;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Param;

/**
 * 用户表 数据层
 * 
 * @author dgtly
 */
public interface SysUserMapper
{


    /**
     * 查询所有用户LoginName
     *
     * @return 用户信息集合信息
     */
    public Set<String> selectAllUserLoginName();

    public Set<String> selectDiyUserLoginName();

    /**
     * 根据条件分页查询用户列表
     * 
     * @param sysUser 用户信息
     * @return 用户信息集合信息
     */
    public List<SysUser> selectUserList(SysUser sysUser);

    /**
     * 根据条件分页查询未已配用户角色列表
     * 
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    public List<SysUser> selectAllocatedList(SysUser user);

    /**
     * 根据条件分页查询未分配用户角色列表
     * 
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    public List<SysUser> selectUnallocatedList(SysUser user);

    /**
     * 通过用户名查询用户
     * 
     * @param userName 用户名
     * @return 用户对象信息
     */
    public SysUser selectUserByLoginName(String userName);

    /**
     * 通过手机号码查询用户
     * 
     * @param phoneNumber 手机号码
     * @return 用户对象信息
     */
    public SysUser selectUserByPhoneNumber(String phoneNumber);

    /**
     * 通过邮箱查询用户
     * 
     * @param email 邮箱
     * @return 用户对象信息
     */
    public SysUser selectUserByEmail(String email);

    /**
     * 通过用户ID查询用户
     *
     * @param userId 用户ID
     * @return 用户对象信息
     */
    public SysUser selectUserById(Long userId);

    /**
     * 通过用户ID删除用户
     * 
     * @param userId 用户ID
     * @return 结果
     */
    public int deleteUserById(Long userId);

    /**
     * 批量删除用户信息
     * 
     * @param ids 需要删除的数据ID
     * @return 结果
     */
    public int deleteUserByIds(Long[] ids);

    /**
     * 修改用户信息
     * 
     * @param user 用户信息
     * @return 结果
     */
    public int updateUser(SysUser user);

    //修改认证状态
    public int updateIsAuthentication(@Param("isAuthentication") String isAuthentication,@Param("accountId") String accountId);

    /**
     * 修改全公司的状态
     *
     * @param companyId 公司编号
     * @param status 人员状态：0正常 2删除
     * @return 结果
     */
    public int changeStatus(@Param("companyId") Long companyId,@Param("status")Integer status);

    /**
     * 新增用户信息
     * 
     * @param user 用户信息
     * @return 结果
     */
    public int insertUser(SysUser user);

    /**
     * 新增用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    public int insertUserNoKey(SysUser user);

    /**
     * 校验用户名称是否唯一
     * 
     * @param loginName 登录名称
     * @return 结果
     */
    public int checkLoginNameUnique(@Param("loginName") String loginName);

    /**
     * 校验手机号码是否唯一
     *
     * @param phonenumber 手机号码
     * @return 结果
     */
    public SysUser checkPhoneUnique(@Param("phonenumber") String phonenumber,@Param("companyId") Long companyId);

    /**
     * 校验email是否唯一
     *
     * @param email 用户邮箱
     * @return 结果
     */
    public SysUser checkEmailUnique(@Param("email")String email,@Param("companyId") Long companyId);

    /**
     * 根据立邦员工id查找用户
     *
     * @param emplyeeId 立邦员工id
     * @return 结果
     */
    public SysUser selectUserByEmplyeeId(String emplyeeId);

    /**
     * 根据销售组织代码 获取所属的用户登陆名称
     *
     * @param customerCodes 经销商列表
     * @return 结果
     */
    @MapKey("customer_code")
    Map<String,Map<String,Object>> selectLoginNamesByCostumerCode(String[] customerCodes);

    /**
     * 根据销售组织代码 获取所属的用户登陆名称
     *
     * @param employeeCode 经销商列表
     * @return 结果
     */
    @MapKey("employee_code")
    Map<String,Map<String,Object>> selectLoginNamesByEmployeeCode(String[] employeeCode);

    /**
     * @description: 根据经销商code查询经销人员列表
     * @param: [orgCode]
     * @return: void
     * @author: qxm
     * @date: 2020/10/19 14:36
     */
    List<SysUser> selectUserByOrgCode(String orgCode);

    Map<String,Object> selectLoginNamesByCostumerCode1(String[] customerCodes);

    String getCustomNameByCode(String code);
}
