package com.dgtly.system.service;

import java.util.List;
import java.util.Map;
import java.util.Set;

import com.dgtly.common.exception.BusinessException;
import com.dgtly.system.domain.SysUser;

/**
 * 用户 业务层
 * 
 * @author dgtly
 */
public interface ISysUserService
{



    /**
     * 查询所有用户LoginName
     *
     * @return 用户信息集合信息
     */
    public Set<String> selectAllUserLoginName();

    public Set<String> selectDiyUserLoginName();

    /**
     * 根据条件分页查询用户列表
     * 
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    public List<SysUser> selectUserList(SysUser user);

    /**
     * 根据条件分页查询已分配用户角色列表
     * 
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    public List<SysUser> selectAllocatedList(SysUser user);

    /**
     * 根据条件分页查询未分配用户角色列表
     * 
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    public List<SysUser> selectUnallocatedList(SysUser user);

    /**
     * 通过用户名查询用户
     * 
     * @param userName 用户名
     * @return 用户对象信息
     */
    public SysUser selectUserByLoginName(String userName);

    /**
     * 通过手机号码查询用户
     * 
     * @param phoneNumber 手机号码
     * @return 用户对象信息
     */
    public SysUser selectUserByPhoneNumber(String phoneNumber);

    /**
     * 通过邮箱查询用户
     * 
     * @param email 邮箱
     * @return 用户对象信息
     */
    public SysUser selectUserByEmail(String email);

    /**
     * 通过用户ID查询用户
     * 
     * @param userId 用户ID
     * @return 用户对象信息
     */
    public SysUser selectUserById(Long userId);

    /**
     * 通过用户ID删除用户
     * 
     * @param userId 用户ID
     * @return 结果
     */
    public int deleteUserById(Long userId);

    /**
     * 批量删除用户信息
     * 
     * @param ids 需要删除的数据ID
     * @return 结果
     * @throws Exception 异常
     */
    public int deleteUserByIds(String ids) throws BusinessException;

    /**
     * 保存用户信息
     * 
     * @param user 用户信息
     * @return 结果
     */
    public int insertUser(SysUser user);

    /**
     * 插入用户信息和销售扩展信息
     *
     * @param user 用户信息
     * @return 结果
     */
    public Integer insertUserAndSalesExt(SysUser user);




    /**
     * 保存用户信息
     * 
     * @param user 用户信息
     * @return 结果
     */
    public int updateUser(SysUser user);

    /**
     * 修改用户详细信息
     * 
     * @param user 用户信息
     * @return 结果
     */
    public int updateUserInfo(SysUser user);


    //修改认证状态
    public int updateIsAuthentication(String isAuthentication,String accountId);

    /**
     * 修改用户详细信息和销售相关信息
     *
     * @param user 用户信息
     * @return 结果
     */
    public int updateUserInfoAndSalesExt(SysUser user);
    /**
     * 修改公司员工状态
     * @return 结果
     */
    public int changeStatus(Long companyId,Integer status);

    /**
     * 修改用户密码信息
     * 
     * @param user 用户信息
     * @return 结果
     */
    public int resetUserPwd(SysUser user);

    /**
     * 校验用户名称是否唯一
     * 
     * @param loginName 登录名称
     * @return 结果
     */
    public String checkLoginNameUnique(String loginName);

    /**
     * 校验手机号码是否唯一
     *
     * @param user 用户信息
     * @return 结果
     */
    public String checkPhoneUnique(SysUser user);

    /**
     * 校验email是否唯一
     *
     * @param user 用户信息
     * @return 结果
     */
    public String checkEmailUnique(SysUser user);

    /**
     * 校验用户是否允许操作
     * 
     * @param user 用户信息
     */
    public void checkUserAllowed(SysUser user);

    /**
     * 根据用户ID查询用户所属角色组
     * 
     * @param userId 用户ID
     * @return 结果
     */
    public String selectUserRoleGroup(Long userId);

    /**
     * 根据用户ID查询用户所属岗位组
     * 
     * @param userId 用户ID
     * @return 结果
     */
    public String selectUserPostGroup(Long userId);

    /**
     * 导入用户数据
     * 
     * @param userList 用户数据列表
     * @param isUpdateSupport 是否更新支持，如果已存在，则进行更新数据
     * @param operName 操作用户
     * @return 结果
     */
    public String importUser(List<SysUser> userList, Boolean isUpdateSupport, String operName,Long companyId);

    /**
     * 用户状态修改
     * 
     * @param user 用户信息
     * @return 结果
     */
    public int changeStatus(SysUser user);

    /**
     * 根据立邦员工id查找用户
     *
     * @param emplyeeId 立邦员工id
     * @return 结果
     */
    SysUser selectUserByEmplyeeId(String emplyeeId);

    /**
     * 根据销售组织代码 获取所属的用户登陆名称
     *
     * @param customerCodes 经销商
     * @return 结果
     */
    Map<String,Map<String,Object>> selectLoginNamesByCostumerCode(Set<String> customerCodes);

    /**
     * 根据销售的 员工号获取登陆敏成set
     * @param employeeSet
     * @return
     */
    Map<String,Map<String,Object>> selectLoginNamesByEmployeeCode(Set<String> employeeSet);

    public int insertPersonalAccount(SysUser user);

    /**
     * @description: 根据经销商code查询经销人员列表
     * @param: [orgCode]
     * @return: java.util.List<com.dgtly.system.domain.SysUser>
     * @author: qxm      
     * @date: 2020/10/19 14:38
     */ 
    List<SysUser> selectUserByOrgCode(String orgCode);

    String getCustomNameByCode(String code);
}
