package com.dgtly.system.service.impl;

import java.util.List;
import com.dgtly.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.dgtly.system.mapper.AssRelcustomerinfoMapper;
import com.dgtly.system.domain.AssRelcustomerinfo;
import com.dgtly.system.service.IAssRelcustomerinfoService;
import com.dgtly.common.core.text.Convert;

/**
 * 多经销商关联Service业务层处理
 * 
 * @author dgtly
 * @date 2021-04-08
 */
@Service
public class AssRelcustomerinfoServiceImpl implements IAssRelcustomerinfoService 
{
    @Autowired
    private AssRelcustomerinfoMapper assRelcustomerinfoMapper;

    /**
     * 查询多经销商关联
     * 
     * @param id 多经销商关联ID
     * @return 多经销商关联
     */
    @Override
    public AssRelcustomerinfo selectAssRelcustomerinfoById(Long id)
    {
        return assRelcustomerinfoMapper.selectAssRelcustomerinfoById(id);
    }

    /**
     * 查询多经销商关联列表
     * 
     * @param assRelcustomerinfo 多经销商关联
     * @return 多经销商关联
     */
    @Override
    public List<AssRelcustomerinfo> selectAssRelcustomerinfoList(AssRelcustomerinfo assRelcustomerinfo)
    {
        return assRelcustomerinfoMapper.selectAssRelcustomerinfoList(assRelcustomerinfo);
    }

    /**
     * 新增多经销商关联
     * 
     * @param assRelcustomerinfo 多经销商关联
     * @return 结果
     */
    @Override
    public int insertAssRelcustomerinfo(AssRelcustomerinfo assRelcustomerinfo)
    {
        assRelcustomerinfo.setCreateTime(DateUtils.getNowDate());
        return assRelcustomerinfoMapper.insertAssRelcustomerinfo(assRelcustomerinfo);
    }

    /**
     * 修改多经销商关联
     * 
     * @param assRelcustomerinfo 多经销商关联
     * @return 结果
     */
    @Override
    public int updateAssRelcustomerinfo(AssRelcustomerinfo assRelcustomerinfo)
    {
        assRelcustomerinfo.setUpdateTime(DateUtils.getNowDate());
        return assRelcustomerinfoMapper.updateAssRelcustomerinfo(assRelcustomerinfo);
    }

    /**
     * 删除多经销商关联对象
     * 
     * @param ids 需要删除的数据ID
     * @return 结果
     */
    @Override
    public int deleteAssRelcustomerinfoByIds(String ids)
    {
        return assRelcustomerinfoMapper.deleteAssRelcustomerinfoByIds(Convert.toStrArray(ids));
    }

    /**
     * 删除多经销商关联信息
     * 
     * @param id 多经销商关联ID
     * @return 结果
     */
    @Override
    public int deleteAssRelcustomerinfoById(Long id)
    {
        return assRelcustomerinfoMapper.deleteAssRelcustomerinfoById(id);
    }

    @Override
    public List<AssRelcustomerinfo> selectRelcustomerinfoListByorgCode(String orgCode) {
        return assRelcustomerinfoMapper.selectRelcustomerinfoListByorgCode(orgCode);
    }

    @Override
    public void relationCustomerOnlineSync() {
        assRelcustomerinfoMapper.relationCustomerOnlineSync();
    }
}
