package com.dgtly.wxportal.controller;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.dgtly.common.annotation.ApiPassToken;
import com.dgtly.common.annotation.Log;
import com.dgtly.common.core.controller.ApiBaseController;
import com.dgtly.common.core.domain.AjaxResult;
import com.dgtly.common.core.domain.ParameterObject;
import com.dgtly.common.enums.BusinessType;
import com.dgtly.system.domain.*;
import com.dgtly.system.service.*;
import com.dgtly.wxportal.domain.HelpCenter;
import com.dgtly.wxportal.domain.WxBanner;
import com.dgtly.wxportal.domain.WxMagnet;
import com.dgtly.wxportal.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;


@Api(tags = "微信门户相关接口")
@RestController
@RequestMapping("/wxportal")
@ApiPassToken
public class WxPortalController extends ApiBaseController {

    @Autowired
    private IWxMagnetService wxMagnetService;

    @Autowired
    private IWxBannerService wxBannerService;
    @Autowired
    private ESignService eSignService;
    @Autowired
    private ISysUserService userService;
    @Autowired
    private ISysUserExtService userExtService;
    @Autowired
    private ICustomersExtService customersExtService;
    @Autowired
    private IEsignPersonRealnVerifyService personRealnVerifyService;
    @Autowired
    private IEsignOrganRealVerifyService organRealVerifyService;
    @Autowired
    private IHelpCenterService helpCenterService;
    @Autowired
    private IAssRelcustomerinfoService assRelcustomerinfoService;
    @Autowired
    private ISysMagnetLogService sysMagnetLogService;


    @ApiOperation(value = "微信门户首页",notes = "参数：{userId:1}")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "params" , paramType = "body")
    })
    @PostMapping("/portalData")
    public Object getMagent(){
        ParameterObject obj =  getParameterObject();
        obj.checkParameterNotNull("userId");
        WxBanner wxBanner = new WxBanner();
        List<WxMagnet> wxMagnetList =  wxMagnetService.selectMagnetByUserId(obj.getString("userId"), obj.getString("cw"), obj.getString("salesLevel"));
        List<WxBanner> wxBannerList = wxBannerService.selectWxBannerList(wxBanner);
        return AjaxResult.success().putKV("magnets",wxMagnetList).putKV("banners",wxBannerList);
    }

    /**
     * 新增保存磁铁日志
     */
    @ApiOperation(value = "新增磁铁访问日志",notes = "参数：{userId,userName,orgCode,orgName,level,magnetName}")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "params" , paramType = "body")
    })
    @PostMapping("/magnetLogAdd")
    @ResponseBody
    public AjaxResult magnetLogAdd()
    {
        ParameterObject obj = getParameterObject();
        obj.checkParameterNotNull("userId,userName,level,magnetName");
        SysMagnetLog sysMagnetLog = new SysMagnetLog();
        sysMagnetLog.setUserId(obj.getLong("userId"));
        sysMagnetLog.setUserName(obj.getString("userName"));
        sysMagnetLog.setOrgCode(obj.getString("orgCode"));
        sysMagnetLog.setOrgName(obj.getString("orgName"));
        sysMagnetLog.setLevel(obj.getString("level"));
        sysMagnetLog.setMagnetName(obj.getString("magnetName"));
        return toAjax(sysMagnetLogService.insertSysMagnetLog(sysMagnetLog));
    }

    @ApiOperation(value = "查询经销商人员列表", notes = "参数：{'userId':'XXXXXXX'}" +
            "\n（userId：用户id）")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "params", paramType = "body")
    })
    @PostMapping("getDistributorList")
    public Object getDistributorList() {
        ParameterObject obj = getParameterObject();
        obj.checkParameterNotNull("userId");
        String userId = obj.getString("userId");
        SysUserExt userExt = userExtService.selectSysUserExtById(Long.parseLong(userId));
        if (userExt!=null){
            List<SysUser> userList = userService.selectUserByOrgCode(userExt.getOrgCode());
            return AjaxResult.success().putKV("userList",userList);
        }else{
            return AjaxResult.error("用户不存在");
        }
    }

    @ApiOperation(value = "查询多经销商列表", notes = "参数：{'userId':'XXXXXXX'}" +
            "\n（userId：用户id）")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "params", paramType = "body")
    })
    @PostMapping("getSysUserExtList")
    public Object getSysUserExtList() {
        ParameterObject obj = getParameterObject();
        obj.checkParameterNotNull("userId");
        String userId = obj.getString("userId");
        String customerCode = obj.getString("customerCode");
        Map map = new HashMap(5);
        SysUser sysUser = userService.selectUserById(Long.parseLong(userId));
        /*根据org_code查询多个经销商列表*/
        List<SysUserExt> sysUserExtList = new ArrayList<>();
        List<AssRelcustomerinfo>  assRelcustomerinfoList= assRelcustomerinfoService.selectRelcustomerinfoListByorgCode(customerCode);
        if (assRelcustomerinfoList.size()>0){
            for (AssRelcustomerinfo ar:assRelcustomerinfoList) {
                SysUserExt sysUserExt = new SysUserExt();
                sysUserExt.setUserId(sysUser.getSysUserExt().getUserId());
                sysUserExt.setOrgCode(ar.getKunnr());
                sysUserExt.setOrgName(ar.getName1());
                sysUserExt.setSalesLevel(sysUser.getSysUserExt().getSalesLevel());
                sysUserExt.setBossEmployeeId(sysUser.getSysUserExt().getBossEmployeeId());
                sysUserExt.setBossName(sysUser.getSysUserExt().getBossName());
                sysUserExt.setSapEmployeeId(sysUser.getSysUserExt().getSapEmployeeId());
                sysUserExt.setCustomerCode(sysUser.getSysUserExt().getCustomerCode());
                sysUserExt.setIsCustomerManager(sysUser.getSysUserExt().getIsCustomerManager());
                sysUserExt.setPostCode(sysUser.getSysUserExt().getPostCode());
                sysUserExt.setPostName(sysUser.getSysUserExt().getPostName());
                sysUserExtList.add(sysUserExt);
            }
            SysUserExt sysUserExt = new SysUserExt();
            sysUserExt.setUserId(sysUser.getSysUserExt().getUserId());
            sysUserExt.setOrgCode(assRelcustomerinfoList.get(0).getMainkunnr());
            sysUserExt.setOrgName(assRelcustomerinfoList.get(0).getMainname1());
            sysUserExt.setSalesLevel(sysUser.getSysUserExt().getSalesLevel());
            sysUserExt.setBossEmployeeId(sysUser.getSysUserExt().getBossEmployeeId());
            sysUserExt.setBossName(sysUser.getSysUserExt().getBossName());
            sysUserExt.setSapEmployeeId(sysUser.getSysUserExt().getSapEmployeeId());
            sysUserExt.setCustomerCode(sysUser.getSysUserExt().getCustomerCode());
            sysUserExt.setIsCustomerManager(sysUser.getSysUserExt().getIsCustomerManager());
            sysUserExt.setPostCode(sysUser.getSysUserExt().getPostCode());
            sysUserExt.setPostName(sysUser.getSysUserExt().getPostName());
            sysUserExtList.add(sysUserExt);
        }
        sysUser.setSysUserExtLsit(sysUserExtList);
        if (sysUser!=null){
//            map.put("user",sysUser);
            map.put("sysUserExtList",sysUserExtList);
            return AjaxResult.success(map);
        }else{
            return AjaxResult.error("用户不存在");
        }
    }

    @ApiOperation(value = "切换经销商", notes = "参数：{'userId':'XXXXXXX','orgCode':'XXXXXXX','orgName':'XXXXXXX'}" +
            "\n（userId：用户id）" +
            "\n（orgCode：经销商code）" +
            "\n（orgName：经销商名称）")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "params", paramType = "body")
    })
    @PostMapping("switchCustomer")
    public Object switchCustomer() {
        ParameterObject obj = getParameterObject();
        obj.checkParameterNotNull("userId,orgCode,orgName");
        String userId = obj.getString("userId");
        String orgCode = obj.getString("orgCode");
        String orgName = obj.getString("orgName");

        //check
        SysUser sysUser = userService.selectUserById(Long.parseLong(userId));
        if (sysUser != null && "customer_level".equals(sysUser.getSysUserExt().getSalesLevel())) {
            int row = userExtService.switchCustomer(userId, orgCode, orgName);
            if (row > 0) {
                //通过userId查询用户信息
                SysUser user = userService.selectUserById(Long.valueOf(userId));
                return AjaxResult.success().putKV("sysUser", user);
            } else {
                return AjaxResult.error("经销商切换失败");
            }
        } else {
            return AjaxResult.error(301,"非经销商级别无需切换");
        }
    }

    @ApiOperation(value = "设置/取消是否为收货人", notes = "参数：{'userId':'XXXXXXX','IsConsignee':'1'}" +
            "\n（userId：用户id" +
            "\n IsConsignee:是否收货人（0否1是））")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "params", paramType = "body")
    })
    @PostMapping("updateOrgUser")
    public Object updateOrgUser() {
        ParameterObject obj = getParameterObject();
        obj.checkParameterNotNull("userId,IsConsignee");
        String userId = obj.getString("userId");
        String IsConsignee = obj.getString("IsConsignee");
        SysUser user = new SysUser();
        user.setUserId(Long.parseLong(userId));
        user.setIsConsignee(IsConsignee);
        int i = userService.updateUserInfo(user);
        if (i>0){
            return AjaxResult.success();
        }else {
            return AjaxResult.error();
        }
    }

    @ApiOperation(value = "帮助中心",notes = "参数：{userId:1}")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "params" , paramType = "body")
    })
    @PostMapping("/helpCenter")
    public Object getHelpCenter() {
        ParameterObject obj =  getParameterObject();
        obj.checkParameterNotNull("userId");
        HelpCenter helpCenter = new HelpCenter();
        List<HelpCenter> list = helpCenterService.selectHelpCenterList(helpCenter);
        return AjaxResult.success().putKV("helpList",list);
    }
}
